/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#include "itemfactory.h"

#include <QVApplication>
#include <QVMPlayerCamera>
#include <QVWorker>
#include "qvworkers/qvfilterselectorworker.h"
#include "qvworkers/qvimageretarderworker.h"
#include "qvworkers/qvimagemeansworker.h"
#include "qvworkers/qvgraphsworker.h"
#include "qvworkers/qvpointworker.h"
#include <QVImageCanvas>
#include <QVNumericPlot>
#include <QVHistogramPlot>
#include <QVCPUPlot>

///@todo no crear todos los workers aqui, sino sacal la lista de nombres y luego crearlos bajo demanda
ItemFactory::ItemFactory(): QObject()
{
	inputItems.append("QVMPlayerCamera");
	inputItems.append("QVPointWorker");

	outputItems.append("QVImageCanvas");
	outputItems.append("QVNumericPlot");
	outputItems.append("QVHistogramPlot");
	outputItems.append("QVCPUPlot");
	middleItems.append("QVGraphsWorker");
	middleItems.append("QVFilterSelectorWorker<uchar,1>");
	middleItems.append("QVFilterSelectorWorker<uchar,3>");
	middleItems.append("QVImageRetarderWorker<uchar,3>");
	middleItems.append("QVImageRetarderWorker<uchar,1>");
	middleItems.append("QVImageMeans<uChar,1>");
	middleItems.append("QVImageMeans<uChar,3>");

	// Añadimos los tipos de las IPP.
	foreach(QString qvippWorkerName, getQVIPPWorkersNames())
		middleItems.append(qvippWorkerName);

	// Añadimos los tipos creados por el usuario.
/*	foreach(QVPropertyContainer* qvp, qvApp->getQVPropertyContainers()) {
		QVWorker* worker;
		if((worker = dynamic_cast<QVWorker*>(qvp)) != NULL)
			middleItems.append(worker.getName());
		}*/
}

QList<QString> ItemFactory::getItemTypes() const
{
	return inputItems + middleItems + outputItems;
}

QList<QString> ItemFactory::getInputItemTypes() const
{
	return inputItems;
}

QList<QString> ItemFactory::getMiddleItemTypes() const
{
	return middleItems;
}

QList<QString> ItemFactory::getOutputItemTypes() const
{
	return outputItems;
}

QVPropertyContainer *ItemFactory::createContainer(QString type, QString name) const
{
	QVPropertyContainer *qvippWorker = newQVIPPWorker(type, name);
	if (qvippWorker != NULL)
		return qvippWorker;
	else if (type == "QVMPlayerCamera") {
		QVMPlayerCamera* mpc = new QVMPlayerCamera(name);
		if (mpc->getPropertyValue<QString>("URL").isEmpty())
			mpc->setPropertyValue("URL", QString("http://perception.inf.um.es/public_data/videos/misc/penguin.dv"));
		return mpc;
	}
	else if (type == "QVImageCanvas") return new QVImageCanvas(name);
	else if (type == "QVNumericPlot") return new QVNumericPlot(name);
	else if (type == "QVHistogramPlot") return new QVHistogramPlot(name);
	else if (type == "QVCPUPlot") return new QVCPUPlot(name);
	else if (type == "QVGraphsWorker") return new QVGraphsWorker(name);
	else if (type == "QVFilterSelectorWorker<uchar,1>") return new QVFilterSelectorWorker<uchar, 1>(name);
	else if (type == "QVFilterSelectorWorker<uchar,3>") return new QVFilterSelectorWorker<uchar, 3>(name);
	else if (type == "QVImageRetarderWorker<uchar,3>") return new QVImageRetarderWorker<uchar, 3>(name);
	else if (type == "QVImageRetarderWorker<uchar,1>") return new QVImageRetarderWorker<uchar, 1>(name);
	else if (type == "QVImageMeans<uChar,1>") return new QVImageMeans<uChar, 1>(name);
	else if (type == "QVImageMeans<uChar,3>") return new QVImageMeans<uChar, 3>(name);
	else if (type == "QVPointWorker") return new QVPointWorker(name);
	else if (userWorkers.contains(type)) {
		QVWorker * newWorker = (QVWorker *)QMetaType::construct(userWorkers.value(type), (void *)0);
		newWorker->setName(name);
		return newWorker;
	}
	return 0;
}

bool ItemFactory::deleteContainer(QVPropertyContainer *cont) const
{
	// obtiene el nombre y el valor del tipo de la subclase concreta
	QString type = typeName(cont);

/*	// lo destruye dependiendo de como se ha creado
	if (isUserType(type))
		QMetaType::destroy(userWorkers.value(type), (void *)cont);
	else
		delete cont;*/

	delete cont;

	return TRUE;
}

QString ItemFactory::containerType(QVPropertyContainer *cont) const
{
	// obtiene el nombre y el valor del tipo de la subclase concreta
	return QString(typeName(cont));
}

const QString ItemFactory::registerUserWorker(QVWorker * worker)
{
	const char * choptypeidname = typeName(worker);

	// obtiene el nombre y el valor del tipo de la subclase concreta
	QString typeString(choptypeidname);
	int type = QMetaType::type(choptypeidname);

	// almacena el subtipo concreto del objeto, si no existía ya y si estaba registrado en el sistema con qRegisterMetaType<Type>("Type")
	if ( (type != 0) && (!middleItems.contains(typeString)) ) {
		middleItems.append(typeString);
		userWorkers.insert(typeString, type);
		return typeString;
	}
	return QString();
}

bool ItemFactory::isUserType(QString type) const
{
	return userWorkers.contains(type);
}

const char *ItemFactory::typeName(QVPropertyContainer *cont) const
{
	// obtiene el tipo real del objeto (la sunclase concreta de worker) en un formato char * de longitud+nombre (ejemplo: 8MyWorker)
	const char * typeidname = typeid(*cont).name();

	// se salta la pirmera parte de la cadena, la que contiene el número de caracteres del tipo
	int i = 0;
	while ( (typeidname[i] != '\0') && ((typeidname[i] == '0') || (typeidname[i] == '1') || (typeidname[i] == '2') || (typeidname[i] == '3') ||
	        (typeidname[i] == '4') || (typeidname[i] == '5') || (typeidname[i] == '6') || (typeidname[i] == '7') || (typeidname[i] == '8') ||
	        (typeidname[i] == '9')) ) {
		i++;
	}
	return (typeidname + i*(sizeof(char)));
}


