/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


//#include "qvdesignergui.h"
#include <QVDesignerGUI>
#include <QVApplication>
#include <QVPropertyContainer>
#include <QVPropertyContainerChange>
#include <QVMPlayerCamera>
#include <QVWorker>
#include <QVImageCanvas>
#include <qvgui/qvplot.h>
#include <qvgui/qvdesigner/facade/workerdialog.h>
#include <qvgui/qvdesigner/facade/cameradialog.h>


QVDesignerGUI::QVDesignerGUI(): QObject(), factory(), slate(this), informer(&slate)
{
	qDebug() << "QVDesignerGUI::QVDesignerGUI()";
	if (qvApp == NULL)
		{
		QString str =	"QVDesignerGUI::QVDesignerGUI(): the SlateGUI cannot be created before the QVApplication instance. Aborting now.";
		std::cerr << qPrintable(str) << std::endl;
		exit(1);
		}

	// if its a --help call, do nothing
	if (qvApp->forHelp()) return;

	connect(&slate, SIGNAL(closed()), this, SLOT(quit()));

	qvApp->registerGUI(this);
	qvApp->setTerminateOnLastWorker(false); // para que no termine cuando se resetea el escenario (y se quitan todos los workers temporalmente)

/*	// se conecta al informador estatico de la clase para obtener señales cuando se crea y se destruye un property container
	QVPropertyContainerInformer *inf = QVPropertyContainer::getGlobalInformer();
	connect(inf, SIGNAL(changed(QVPropertyContainerChange)), this, SLOT(processChange(QVPropertyContainerChange)));*/

	qDebug() << "QVDesignerGUI::QVDesignerGUI() <~ return";
}

void QVDesignerGUI::init()
{
	// registra y pinta los property containers
	foreach(QVPropertyContainer* qvp, qvApp->getQVPropertyContainers()) {
		// registro su tipo
		QVWorker* worker;
		if((worker = dynamic_cast<QVWorker*>(qvp)) != NULL) {
			QString registered = factory.registerUserWorker(worker);
			if (!registered.isEmpty())
				slate.includeItemType(registered);
		}

		// registra y pinta el property container
		QString type = factory.containerType(qvp);
		QString name = qvp->getName();
		// no se incluye en "containers", ya que no se pueden borrar desde aqui, ya que se crearon en el main y no con new. Se registra en "initialContainers"
		initialContainers.insert(name, qvp);
		createdItems.insert(name, CreatedItem(type, name));
		// se conecta al informador de ese container para obtener señales de sus cambios.
		connect(qvp->getInformer(), SIGNAL(changed(QVPropertyContainerChange)), this, SLOT(processChange(QVPropertyContainerChange)));
		slate.addItemNode(type, name, new ItemProperties(type, qvp));
		// creo su dialogo
		createDialog(name);
	}

	// una vez he registrado todos los property containers, registro y pinto los links de salida de cada uno (asi los pinto todos sin repetir)
	foreach(QVPropertyContainer* qvp, qvApp->getQVPropertyContainers()) {
		foreach(QString prop, qvp->getPropertyList()) {
			QVPropertyContainer *source = qvp->getSourceContainer(prop);
			QString sourceProp = qvp->getSourceProperty(prop);
			bool sync = qvp->isSynchronous(prop);

			if ( (source != NULL) && (!sourceProp.isEmpty()) ) {
				QString fromName = source->getName();
				QString fromProp = sourceProp;
				QString toName = qvp->getName();
				QString toProp = prop;
				QString linkName = getLinkName(fromName, fromProp, toName, toProp);
				createdLinks.insert(linkName, CreatedLink(fromName, fromProp, toName, toProp, sync));
				slate.addLinkLine(fromName, fromProp, toName, toProp, sync);
// 				informer.inform(QString("Link añadido: ") + getLinkName(fromName, fromProp, toName, toProp));
			}
		}
	}

	// por último se pide a la vista, ahora que estan todos los nodos y links, que reordene los mismos
	slate.arrangeItems();
}

void QVDesignerGUI::show()
{
	slate.show();
}

QList<QString> QVDesignerGUI::getItemTypes() const
{
	return factory.getItemTypes();
}

QList<QString> QVDesignerGUI::getInputItemTypes() const
{
	return factory.getInputItemTypes();
}

QList<QString> QVDesignerGUI::getMiddleItemTypes() const
{
	return factory.getMiddleItemTypes();
}

QList<QString> QVDesignerGUI::getOutputItemTypes() const
{
	return factory.getOutputItemTypes();
}

///@todo cuidado con los nombres repetidos y/o vacios
bool QVDesignerGUI::addItem(QString type, QString name)
{
	if (containers.contains(name)) return FALSE;

	//std::cout << "Creo " + name.toStdString() + "\n";
	QVPropertyContainer *cont = factory.createContainer(type, name);
	if (cont != 0) {
		containers.insert(name, cont);
		createdItems.insert(name, CreatedItem(type, name));
		// se conecta al informador de ese container para obtener señales de sus cambios.
		connect(cont->getInformer(), SIGNAL(changed(QVPropertyContainerChange)), this, SLOT(processChange(QVPropertyContainerChange)));

		slate.addItemNode(type, name, new ItemProperties(type, cont));
		return TRUE;
	}
	return FALSE;
}

bool QVDesignerGUI::addLink(QString fromName, QString fromProp, QString toName, QString toProp, bool synchronous)
{
	///@todo homogeneizar un poco las llamadas a link de los distintops tipos de elementos
	if (containers.contains(fromName) && containers.contains(toName)) {
		QVPropertyContainer *fromCont = containers.value(fromName);
		QVPropertyContainer *toCont = containers.value(toName);

		QVMPlayerCamera* camera;
		QVWorker* worker;
		QVWorker* worker2;
		QVImageCanvas* canvas;
		QVPlot* plot;

		if((camera = dynamic_cast<QVMPlayerCamera*>(fromCont)) != NULL) {
			if((worker = dynamic_cast<QVWorker*>(toCont)) != NULL) {
				if (!synchronous) { // solo se permite AsynchronousLink
					camera->link(worker, toProp);
				}
				else informer.inform("Una camara solo se puede lincar de forma asincrona");
			}
		}
		else if((worker = dynamic_cast<QVWorker*>(fromCont)) != NULL) {
			if((worker2 = dynamic_cast<QVWorker*>(toCont)) != NULL) {
				if (synchronous) {
					worker->linkProperty(fromProp, worker2, toProp, QVWorker::SynchronousLink);
				}
				else {
					worker->linkProperty(fromProp, worker2, toProp, QVWorker::AsynchronousLink);
				}
			}
			else if((canvas = dynamic_cast<QVImageCanvas*>(toCont)) != NULL) {
				if (!synchronous) { // solo se permite AsynchronousLink
					canvas->linkProperty(*worker, fromProp);
				}
				else informer.inform("Un canvas solo se puede lincar de forma asincrona");
			}
			else if((plot = dynamic_cast<QVPlot*>(toCont)) != NULL) {
				if (!synchronous) { // solo se permite AsynchronousLink
					plot->linkProperty(*worker, fromProp);
				}
				else informer.inform("Un plot solo se puede lincar de forma asincrona");
			}
		}
	}
	return TRUE;
}

bool QVDesignerGUI::delItem(const QString name)
{
	if (containers.contains(name)) {
		// borramos el propertyContainer asociado
		factory.deleteContainer(containers.value(name));
		return true;
	}
	return false;
}

bool QVDesignerGUI::delLink(const QString fromName, const QString fromProp, const QString toName, const QString toProp)
{
	if (containers.contains(fromName) && containers.contains(toName)) {
		QVPropertyContainer *fromCont = containers.value(fromName);
		QVPropertyContainer *toCont = containers.value(toName);

		QVMPlayerCamera* camera;
		QVWorker* worker;
		QVWorker* worker2;
		QVImageCanvas* canvas;
		QVPlot* plot;

		if((camera = dynamic_cast<QVMPlayerCamera*>(fromCont)) != NULL) {
			if((worker = dynamic_cast<QVWorker*>(toCont)) != NULL) {
				camera->unlink(worker, toProp);
			}
		}
		else if((worker = dynamic_cast<QVWorker*>(fromCont)) != NULL) {
			if((worker2 = dynamic_cast<QVWorker*>(toCont)) != NULL) {
				worker->unlinkProperty(fromProp, worker2, toProp);
			}
			else if((canvas = dynamic_cast<QVImageCanvas*>(toCont)) != NULL) {
				canvas->unlink(worker, fromProp);
			}
			else if((plot = dynamic_cast<QVPlot*>(toCont)) != NULL) {
				plot->unlink(worker, fromProp);
			}
		}
	}
	return TRUE;
}

template <class Type> bool QVDesignerGUI::setProperty(const QString fromName, const QString fromProp, const Type &value)
{
	return false;
}

QString QVDesignerGUI::getLinkName(QVPropertyContainerChange change)
{
	return getLinkName(change.getOrigName(), change.getOrigProp(), change.getDestName(), change.getDestProp());
}

QString QVDesignerGUI::getLinkName(QString fromName, QString fromProp, QString toName, QString toProp)
{
	return getAbsPropName(fromName, fromProp) + " => " + getAbsPropName(toName, toProp);
}

QString QVDesignerGUI::getAbsPropName(QString fromName, QString fromProp)
{
	return fromName + "[" + fromProp + "]";
}


void QVDesignerGUI::processChange(QVPropertyContainerChange change)
{
	switch (change.getChangeType())
	{
	case QVPropertyContainerChange::Name:
		if (createdItems.contains(change.getSourceName()))
			;//std::cout << change.toString().toStdString() << std::endl;
		break;

/*	case QVPropertyContainerChange::ContainerAdd:
		std::cout << change.toString().toStdString() << std::endl;
		break;*/

	case QVPropertyContainerChange::ContainerDel:
		{
			QString name = change.getSourceName();
			containers.remove(name);
			createdItems.remove(name);
			slate.delItemNode(name);
		
			// si se ha creado su dialogo, tambien lo borramos
			deleteDialog(name);

			//std::cout << change.toString().toStdString() << std::endl;
		}
		break;

	case QVPropertyContainerChange::PropertyAdd:
		if (createdItems.contains(change.getSourceName()))
		{
			QString srcName = change.getSourceName();
			QString propName = change.getPropName();

			if (containers.contains(srcName))
			{
				QVPropertyContainer *cont = containers.value(srcName);
				int type = cont->getPropertyType(propName);
				bool in = cont->isInput(propName);
				bool out = cont->isOutput(propName);
				slate.addProperty(srcName, propName, type, in, out);
			}
			//std::cout << change.toString().toStdString() << std::endl;
		}
		break;

	case QVPropertyContainerChange::PropertyDel:
		if (createdItems.contains(change.getSourceName()))
		{
			QString srcName = change.getSourceName();
			QString propName = change.getPropName();
			slate.delProperty(srcName, propName);
			//std::cout << change.toString().toStdString() << std::endl;
		}
		break;

	case QVPropertyContainerChange::PropertyValue:
		break;

	case QVPropertyContainerChange::LinkAdd:
		{
			QString fromName = change.getOrigName();
			QString fromProp = change.getOrigProp();
			QString toName = change.getDestName();
			QString toProp = change.getDestProp();
			QString linkName = getLinkName(change);
			if ( createdItems.contains(change.getOrigName()) && createdItems.contains(change.getDestName()) )
			{
				createdLinks.insert(linkName, CreatedLink(fromName, fromProp, toName, toProp, change.isSinc()));
				slate.addLinkLine(fromName, fromProp, toName, toProp, change.isSinc());
// 				informer.inform(QString("Link creado: ") + linkName);
			}
// 			else
// 				informer.inform(QString("Se ha creado un link no registrado: ") + linkName);
		}
		break;

	case QVPropertyContainerChange::LinkDel:
		{
			QString fromName = change.getOrigName();
			QString fromProp = change.getOrigProp();
			QString toName = change.getDestName();
			QString toProp = change.getDestProp();
			QString linkName = getLinkName(change);
			if (createdLinks.contains(linkName))
			{
				createdLinks.remove(linkName);
				slate.delLinkLine(fromName, fromProp, toName, toProp);
				//std::cout << change.toString().toStdString() << std::endl;
			}
		}
		break;

	case QVPropertyContainerChange::All:
		//std::cout << change.toString().toStdString() << std::endl;
		break;

	default:
		break;
	}
}
void QVDesignerGUI::dialogChange(QVPropertyContainerChange change)
{
	switch (change.getChangeType())
	{
		case QVPropertyContainerChange::PropertyValue:
		{
			QString cont = change.getSourceName(); cont.remove(0, 28); // para quitar "QVParamsInspectorWidget for "
			QString prop = change.getPropName();
			propertyChanges.insert(getAbsPropName(cont, prop), PropertyChange(cont, prop, change.getValue()));
			break;
		}

		default:
			break;
	}
}

void QVDesignerGUI::showProperties(const QString itemName)
{
	if (containers.contains(itemName)) {
		if (dialogs.contains(itemName)) {
			(dialogs.value(itemName))->show();
		}
		else {
			if (createDialog(itemName))
				(dialogs.value(itemName))->show();
		}
	}
	else if (initialContainers.contains(itemName)) {
		if (dialogs.contains(itemName)) {
			(dialogs.value(itemName))->show();
		}
		else {
			if (createDialog(itemName))
			(dialogs.value(itemName))->show();
		}
	}
}

bool QVDesignerGUI::createDialog(const QString itemName)
{
	QVPropertyContainer *cont;
	if (containers.contains(itemName))
		cont = containers.value(itemName);
	else
		cont = initialContainers.value(itemName);

	QVWorker* worker;
	QVMPlayerCamera* camera;
	if((worker = dynamic_cast<QVWorker*>(cont)) != NULL) {
		WorkerDialog *wdialog = new WorkerDialog(worker);
		dialogs.insert(itemName, wdialog);
		connect(wdialog->getInformer(), SIGNAL(changed(QVPropertyContainerChange)), this, SLOT(dialogChange(QVPropertyContainerChange)));
		return true;
	}
	else if((camera = dynamic_cast<QVMPlayerCamera*>(cont)) != NULL) {
		CameraDialog * cdialog = new CameraDialog(camera);
		dialogs.insert(itemName, cdialog);
		connect(cdialog->getInformer(), SIGNAL(changed(QVPropertyContainerChange)), this, SLOT(dialogChange(QVPropertyContainerChange)));
		return true;
	}
	return false;
}

void QVDesignerGUI::deleteDialog(const QString itemName)
{
	// si se ha creado su dialogo, tambien lo borramos (puede no haberse creado si no se ha dado a run)
	if (dialogs.contains(itemName)) {
		delete dialogs.value(itemName);
		dialogs.remove(itemName);
	}
}

void QVDesignerGUI::run()
{
	// creamos los dialogos que no se han creado aun, ya que después no se puede (porque no se puede lincar)
	foreach (QString itemName, containers.keys()) {
		if (!dialogs.contains(itemName))
			createDialog(itemName);
	}

	// y arrancamos la simulación
	qvApp->startItems();
}

void QVDesignerGUI::stop()
{
	// guardando la descripción de lo que tenía creado
	QList<CreatedItem> lastItems = createdItems.values();
	QList<CreatedLink> lastLinks = createdLinks.values();
	QList<PropertyChange> lastChanges = propertyChanges.values();

	// detenemos la simulación (lo que desregistra los elementos del qvApp y deslinca los items)
	qvApp->quitItems();

	// procesa todos los dellLink emitidos al parar los elementos desde el qvapp (para que no se solapen con su recontrucción)
	qApp->processEvents();

	// oculto los objetos creados desde el main y borro sus dialogos, que se crean al arrancar
	foreach (QVPropertyContainer * cont, initialContainers) {
		QString name = cont->getName();
		slate.delItemNode(name);
		deleteDialog(name);
	}
	initialContainers.clear(); // solo hay que ocultar los elementos de entrada una vez (ya que esos objetos no se pueden eliminar desde este contexto)

	// borro los objetos creados
	foreach (QVPropertyContainer * cont, containers) {
		factory.deleteContainer(cont);
	}

	// creo otros iguales (registradolos en la lista "containers")
	foreach (CreatedItem item, lastItems) {
		if (!addItem(item.type, item.name))
			std::cerr << "QVDesignerGUI - resetItems(): error al crear el Item " + item.name.toStdString() + "\n";
	}

	// creo links iguales a los que teniamos para los nuevos items
	foreach (CreatedLink link, lastLinks) {
		if (!addLink(link.fromName, link.fromProp, link.toName, link.toProp, link.synchronous))
			std::cerr << "QVDesignerGUI - resetItems(): error al crear el Link " + link.fromName.toStdString() + " -> " + link.toName.toStdString() + "\n";
	}

	// restauro los valores cambiados por el usuario
	foreach (PropertyChange change, lastChanges) {
		QString itemName = change.contName;

		if (containers.contains(itemName)) {
			QVPropertyContainer *cont = containers.value(itemName);
			QString prop = change.propName;
			QVariant value = change.value;

			if (cont->isType<int>(prop))
				cont->setPropertyValue<int>(prop, value.value<int>());
			else if (cont->isType<bool>(prop))
				cont->setPropertyValue<bool>(prop, value.value<bool>());
			else if (cont->isType<double>(prop))
				cont->setPropertyValue<double>(prop, value.value<double>());
			else if (cont->isType<QString>(prop))
				cont->setPropertyValue<QString>(prop, value.value<QString>());
			else if (cont->isType<QVIndexedStringList>(prop))
				cont->setPropertyValue<QVIndexedStringList>(prop, value.value<QVIndexedStringList>());
			else
				std::cerr << "QVDesignerGUI - resetItems(): error al restaurar valor: " + getAbsPropName(change.contName, change.propName).toStdString() + "\n";
		}
		else
			std::cerr << "QVDesignerGUI - resetItems(): error al restaurar valor: " + getAbsPropName(change.contName, change.propName).toStdString() + "\n";
	}
}

void QVDesignerGUI::quit()
{
	qvApp->deregisterGUI();
	qvApp->quit();
}

QString QVDesignerGUI::getCppText() const
{
	QString result;

	// incluye una cabezera inicial con el copyright de la librería QVision
	result += "/*\n *	Copyright (C) 2007, 2008. PARP Research Group.\n *	<http://perception.inf.um.es>\n *	University of Murcia, Spain.\n *\n *	This file is part of the QVision library.\n *\n *	QVision is free software: you can redistribute it and/or modify\n *	it under the terms of the GNU Lesser General Public License as\n *	published by the Free Software Foundation, version 3 of the License.\n *\n *	QVision is distributed in the hope that it will be useful,\n *	but WITHOUT ANY WARRANTY; without even the implied warranty of\n *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n *	GNU Lesser General Public License for more details.\n *\n *	You should have received a copy of the GNU Lesser General Public\n *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.\n */\n\n";

	// genera la lista de includes, si alguno de los elementos ha sido definido por el usuario, lo declara para que el usuario lo referencie
	result += "#include <QVApplication>\n#include <QVDefaultGUI>\n";
	QList<QString> types;
	foreach(CreatedItem item, createdItems.values()) {
		QString chopType = item.type;
		int espInd = chopType.indexOf(' ');
		int lesInd = chopType.indexOf('<');
		if ((espInd > 0) && (espInd < lesInd))
			types.append(chopType.left(espInd));
		else
			types.append(chopType.left(lesInd));
	}
	QSet<QString> unrepeatedTypes = types.toSet();
	foreach(QString unrepType, unrepeatedTypes) {
		if (factory.isUserType(unrepType))
			result += QString("class ") + unrepType + QString(";\n");
		else
			result += QString("#include <") + unrepType + QString(">\n");
	}
	// si en alguno de los items se ha modificado una propiedad QVIndexedStringList, se añade su include
	bool QVISLadded = FALSE;
	foreach(CreatedItem item, createdItems.values()) {
		foreach(PropertyChange change, propertyChanges.values()) {
			if ( (change.contName == item.name) && (containers.value(item.name)->isType<QVIndexedStringList>(change.propName)) ) {
				result += QString("#include <QVIndexedStringList>\n");
				QVISLadded = TRUE;
				break;
			}
		}
		if (QVISLadded) break;
	}

	// comienza el método main
	result += "\n#ifndef DOXYGEN_IGNORE_THIS\n\n";
	result += "int main(int argc, char *argv[])\n	{\n	QVApplication app(argc, argv, \"Example program for QVision library.\");\n\n";

	// contruye los objetos usados
	foreach(CreatedItem item, createdItems.values()) {
		result += QString("	") + item.type + QString(" ") + item.name + QString("(\"") + item.name + QString("\");\n");
	}
	result += "\n	QVDefaultGUI interface;\n\n";

	// realiza los links
	foreach(CreatedLink link, createdLinks.values()) {
		if (containers.contains(link.fromName) && containers.contains(link.toName)) {
			QVPropertyContainer *fromCont = containers.value(link.fromName);
			QVPropertyContainer *toCont = containers.value(link.toName);
	
			QVMPlayerCamera* camera;
			QVWorker* worker;
			QVWorker* worker2;
			QVImageCanvas* canvas;
			QVPlot* plot;
	
			if((camera = dynamic_cast<QVMPlayerCamera*>(fromCont)) != NULL) {
				if((worker = dynamic_cast<QVWorker*>(toCont)) != NULL) {
					result += QString("	") + link.fromName + QString(".link(&") + link.toName + QString(", \"") + link.toProp + QString("\");\n");
				}
			}
			else if((worker = dynamic_cast<QVWorker*>(fromCont)) != NULL) {
				if((worker2 = dynamic_cast<QVWorker*>(toCont)) != NULL) {
					result += QString("	") + link.fromName + QString(".linkProperty(\"") + link.fromProp + QString("\", &");
					result += link.toName + QString(", \"") + link.toProp;
					if (link.synchronous)
						result += QString("\", QVWorker::SynchronousLink);\n");
					else
						result += QString("\", QVWorker::AsynchronousLink);\n");
				}
				else if((canvas = dynamic_cast<QVImageCanvas*>(toCont)) != NULL) {
					result += QString("	") + link.toName + QString(".linkProperty(") + link.fromName + QString(", \"") + link.fromProp + QString("\");\n");
				}
				else if((plot = dynamic_cast<QVPlot*>(toCont)) != NULL) {
					result += QString("	") + link.toName + QString(".linkProperty(") + link.fromName + QString(", \"") + link.fromProp + QString("\");\n");
				}
			}
		}
	}

	// establece los valores actuales de las propiedades
	result += "\n";
	foreach(CreatedItem item, createdItems.values()) {
		foreach(PropertyChange change, propertyChanges.values()) {
			if (change.contName == item.name) {
				QVPropertyContainer *cont = containers.value(item.name);
				QString prop = change.propName;
				QVariant value = change.value;
	
				if (cont->isType<int>(prop))
					result += QString("	") + item.name + ".setPropertyValue<int>(\"" + prop + "\", " + value.toString() + ");\n";
				else if (cont->isType<bool>(prop))
					result += QString("	") + item.name + ".setPropertyValue<bool>(\"" + prop + "\", " + value.toString() + ");\n";
				else if (cont->isType<double>(prop))
					result += QString("	") + item.name + ".setPropertyValue<double>(\"" + prop + "\", " + value.toString() + ");\n";
				else if (cont->isType<QString>(prop))
					result += QString("	") + item.name + ".setPropertyValue<QString>(\"" + prop + "\", " + value.toString() + ");\n";
				else if (cont->isType<QVIndexedStringList>(prop)) {
					QVIndexedStringList indlist = value.value<QVIndexedStringList>();
					result += QString("	QVIndexedStringList ") + "QVISLfor_" + item.name + ";\n";
					foreach (QString option, indlist) {
						result += QString("	QVISLfor_") + item.name + ".append(\"" + option + "\");\n";
					}
					result += QString("	QVISLfor_") + item.name + QString(".setIndex(%1);\n").arg(indlist.getIndex());
					result += QString("	") + item.name + ".setPropertyValue<QVIndexedStringList>(\"" + prop + "\", QVISLfor_" + item.name + ");\n";
				}
			}
		}
	}

	// finaliza el fichero
	result += "\n	return app.exec();\n	}\n\n#endif\n\n";
	return result;
}

QString QVDesignerGUI::getXMLText() const
{
	return "Este es el texto\npara el fichero XML\n";
}

