/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ITEMNODE_H
#define ITEMNODE_H

#include "node.h"
#include "../facade/itemproperties.h"

#ifndef DOXYGEN_IGNORE_THIS

class ItemProperties;
class SlateWindow;

class ItemNode : public Node
{

public:
    ItemNode(ItemProperties item, QString _name, SlateWindow *wind, QGraphicsItem * parent = 0, QGraphicsScene * scene = 0);
    ~ItemNode();

    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
	QPointF scenePointPos(int point) const;
	QPointF scenePointPos(QString name, bool in) const;
	void updateLinksPos();
	QString propName(int point) const; // devuelve el nombre de la propiedad señalada por point
	int propPoint(QString name, bool in) const; // devuelve el punto (de entrada o salida) de una propiedad.
	int propType(int point) const; // devuelve el tipo de la propiedad señalada por point
	void insertProperty(int pos, QString name, int type, bool input, bool output);
	void insertProperty(QString name, int type, bool input, bool output);
	void removeProperty(QString name);

protected:
	void mouseDoubleClickEvent(QGraphicsSceneMouseEvent * event);
	void mousePressEvent(QGraphicsSceneMouseEvent * event);
	void mouseMoveEvent(QGraphicsSceneMouseEvent * event);
	void mouseReleaseEvent(QGraphicsSceneMouseEvent * event);
    QVariant itemChange(GraphicsItemChange change, const QVariant &value);

	void markPoint(int point, bool validity);
	void unmarkPoint();
	bool isValidLink(ItemNode *fromNode, int fromPoint, ItemNode *toNode, int toPoint) const;
	void deleteProperty(int pos);
// 	void addNewProp(ItemNode *toNode, int toPoint);
	void delLastProp();

    QRectF outlineRect() const;
    int roundness(double size) const;
	int pointAt(QPointF pos) const;
	virtual int insertPos() const; // posición en que se quieren insertar las nuevas propiedades, para que sea la última menos el los de los "..."

	ItemProperties itemProp; // propiedades del worker asociado
	int numProp; // número de propiedades del worker asociado
	SlateWindow *window; // ventana con que se ha creado, para lanzarle las peticiones de creación de links
	QRectF outlinerect;

	int clickedPoint; // indica si el último click se ha hecho sobre una de los puntos, y sobre cual
	QGraphicsLineItem *line; // linea indicadora del proceso de enlace
	int externalMarkedPoint; // punto de otro item sobre el que se está intentando realizar un enlace
	ItemNode *externalMarkedItem; // item sobre el que se esa intentando realizar el enlace
	int markedPoint; // punto local que se ha seleccionado para colorear por que se está intentando hacer un link sobre el
	bool markedValidity; // indoca si el markedPoint es un objetivo correcto, lo que determinará el color usado
};

#endif
#endif
