/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#include "node.h"
#include <QGraphicsScene>


Node::Node(QGraphicsItem * parent, QGraphicsScene * scene): QGraphicsItem(parent, scene), name(), type() { }

void Node::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
	// actualiza a toda la gerarquía de grupos que tiene por encima, por si han de ensancharse
	QGraphicsItem *ancestor = parentItem();
    while (ancestor) {
		ancestor->update();
		ancestor = ancestor->parentItem();
	}

    QGraphicsItem::mouseMoveEvent(event);
    //if ( parentItem() ) parentItem()->update();
}

void Node::addInLink(Link *link)
{
    myInLinks.append(link);
}

void Node::addOutLink(Link *link)
{
    myOutLinks.append(link);
}

void Node::removeLink(Link *link)
{
    myInLinks.removeAll(link);
	myOutLinks.removeAll(link);
}

QList<Link *> Node::getLinks() const
{
	return (myInLinks + myOutLinks);
}

QList<Link *> Node::getInLinks() const
{
	return myInLinks;
}

QList<Link *> Node::getOutLinks() const
{
	return myOutLinks;
}

int Node::precursors(QList<Node *> tail)
{
	if (myInLinks.isEmpty()) {
		return 0;
	}
	else if (tail.contains(this)) { // si se ha producido un ciclo
		return 0;
	}
	else {
		int maxPre = 0;
		tail.append(this);
		foreach(Link *link, myInLinks) {
			if ((link) && (link->fromNode())) {
				int pre = link->fromNode()->precursors(tail);
				if (maxPre < pre) maxPre = pre; //actualizo maxPre
			}
		}
		return maxPre + 1;
	}
}



void Node::setText(const QString &text)
{
    prepareGeometryChange();
    myText = text;
    update();
}

QString Node::text() const
{
    return myText;
}

void Node::setTextColor(const QColor &color)
{
    myTextColor = color;
    update();
}

QColor Node::textColor() const
{
    return myTextColor;
}

void Node::setOutlineColor(const QColor &color)
{
    myOutlineColor = color;
    update();
}

QColor Node::outlineColor() const
{
    return myOutlineColor;
}

void Node::setBackgroundColor(const QColor &color)
{
    myBackgroundColor = color;
    update();
}

QColor Node::backgroundColor() const
{
    return myBackgroundColor;
}

QRectF Node::boundingRect() const
{
    const int Margin = 1;
    return outlineRect().adjusted(-Margin, -Margin, +Margin, +Margin);
}

QPainterPath Node::shape() const
{
    QRectF rect = outlineRect();

    QPainterPath path;
    path.addRoundRect(rect, roundness(rect.width()),
                      roundness(rect.height()));
    return path;
}

