/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvimagecanvas.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QVImageCanvas>

QVImageCanvas::QVImageCanvas(const QString name, QWidget *parent): QVCanvas(parent), QVPropertyContainer(name), _low(0.0), _high(255.0), circleCenter()
	{
	qDebug() << "QVImageCanvas::QVImageCanvas()";
	if (qvApp == NULL)
		{
		QString str = "QVImageCanvas::QVImageCanvas(): the QVPlot cannot be created before the QVApplication instance. Aborting now.";
		std::cerr << qPrintable(str) << std::endl;
		exit(1);
		}
	// if its a --help call, do nothing
	if (qvApp->forHelp()) return;


	setWindowTitle("Image canvas for " + getName());
	addProperty<QRect>("rect select", outputFlag | internalProp, QRect(), "Rectangule selected in it's image area");
	addProperty<QVPolyline>("poly select", outputFlag | internalProp, QVPolyline(), "Poliline selected in it's image area");
	addProperty<QColor>("poly select $$ color", outputFlag | internalProp, Qt::red, "Color for property poly select"); // to draw the polyline
	addProperty<bool>("poly select $$ printTags", outputFlag | internalProp, false, "Print tag flag poly select");     //
	addProperty<TPolyMode>("poly select $$ polyMode", outputFlag | internalProp, LINE, "Polyline select mode");	    //
	addProperty<QPoint>("poly select $$ center", outputFlag | internalProp, QPoint(), "Polyline select center");	    //
	addProperty<float>("poly select $$ radius", outputFlag | internalProp, 0.0, "Polyline select radius");		    //
	show();
	};

void QVImageCanvas::viewer()
	{
	qDebug() << "QVImageCanvas::viewer()";

	readInputProperties();

	QList<QString> imageNamesC1uChar = getPropertyListByType< QVImage<uChar,1> >();
	if (imageNamesC1uChar.size() == 1)
		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<uChar,1> >(imageNamesC1uChar.at(0)));

	QList<QString> imageNamesC3uChar = getPropertyListByType< QVImage<uChar,3> >();
	if (imageNamesC3uChar.size() == 1)
		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<uChar,3> >(imageNamesC3uChar.at(0)));

	///@todo Las código en qvcanvas para que represente las imagenes uShort, sShort y sInt está incluido pero como genera una imagen negra, cuya razón desconozco aun, he añadido aqui una conversión de esos tipos de imagen a uChar, para su visualización. Lo cual no es lo más correcto correcto, ya que este tipo de conversiones trunca los valores que excedan del rango de uChar (no lo he hecho sobre sFloat porque parece relentizar el proceso). Depurar esto.
	QList<QString> imageNamesC1uShort = getPropertyListByType< QVImage<uShort,1> >();
	if (imageNamesC1uShort.size() == 1)
		{
		QVImage<uChar,1> aux = getPropertyValue<QVImage<uShort,1> >(imageNamesC1uShort.at(0));
		getQVPainter()->drawQVImage(&aux);
//		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<uShort,1> >(imageNamesC1uShort.at(0)));
		}

	///@todo ¿Es correcto hacer la llamada a drawQVImage con un solo parámetro para todos los tipos de imagenes, o para los que se salgan del rango 0-255 habria que hacer uso de: Max(image, max); Min(image, min); getQVPainter()->drawQVImage(&image, TRUE, min, max);?
	QList<QString> imageNamesC3uShort = getPropertyListByType< QVImage<uShort,3> >();
	if (imageNamesC3uShort.size() == 1)
		{
		QVImage<uChar,3> aux = getPropertyValue<QVImage<uShort,3> >(imageNamesC1uShort.at(0));
		getQVPainter()->drawQVImage(&aux);
//		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<uShort,3> >(imageNamesC1uShort.at(0)));
		}

	QList<QString> imageNamesC1sShort = getPropertyListByType< QVImage<sShort,1> >();
	if (imageNamesC1sShort.size() == 1)
		{
		QVImage<uChar,1> aux = getPropertyValue<QVImage<sShort,1> >(imageNamesC1sShort.at(0));
		getQVPainter()->drawQVImage(&aux);
//		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<sShort,1> >(imageNamesC1sShort.at(0)));
		}

	QList<QString> imageNamesC3sShort = getPropertyListByType< QVImage<sShort,3> >();
	if (imageNamesC3sShort.size() == 1)
		{
		QVImage<uChar,3> aux = getPropertyValue<QVImage<sShort,3> >(imageNamesC1sShort.at(0));
		getQVPainter()->drawQVImage(&aux);
//		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<sShort,3> >(imageNamesC1sShort.at(0)));
		}

	QList<QString> imageNamesC1sInt = getPropertyListByType< QVImage<sInt,1> >();
	if (imageNamesC1sInt.size() == 1)
		{
		QVImage<uChar,1> aux = getPropertyValue<QVImage<sInt,1> >(imageNamesC1sInt.at(0));
		getQVPainter()->drawQVImage(&aux);
//		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<sInt,1> >(imageNamesC1sInt.at(0)));
		}

	QList<QString> imageNamesC3sInt = getPropertyListByType< QVImage<sInt,3> >();
	if (imageNamesC3sInt.size() == 1)
		{
		QVImage<uChar,3> aux = getPropertyValue<QVImage<sInt,3> >(imageNamesC1sInt.at(0));
		getQVPainter()->drawQVImage(&aux);
//		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<sInt,3> >(imageNamesC1sInt.at(0)));
		}

	QList<QString> imageNamesC1sFloat = getPropertyListByType< QVImage<sFloat,1> >();
	if (imageNamesC1sFloat.size() == 1)
		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<sFloat,1> >(imageNamesC1sFloat.at(0)),TRUE,_low,_high);

	QList<QString> imageNamesC3sFloat = getPropertyListByType< QVImage<sFloat,3> >();
	if (imageNamesC3sFloat.size() == 1)
		getQVPainter()->drawQVImage(&getPropertyValue<QVImage<sFloat,3> >(imageNamesC3sFloat.at(0)),TRUE,_low,_high);

	foreach(QString name, getPropertyListByType<QVPolyline>())
		{
		TPolyMode mode = containsProperty(name + " $$ polyMode") ? getPropertyValue<TPolyMode>(name + " $$ polyMode") : LINE;
		if (mode == CIRCLE)
			draw(	getPropertyValue<QPoint>(name + " $$ center"),
				getPropertyValue<float>(name + " $$ radius"),
				getPropertyValue<QColor>(name + " $$ color"),
				getPropertyValue<bool>(name + " $$ printTags")
				);
		else
			draw(	getPropertyValue<QVPolyline>(name),
				getPropertyValue<QColor>(name + " $$ color"),
				getPropertyValue<bool>(name + " $$ printTags"),
				mode
				);
		}

	foreach(QString name, getPropertyListByType< QList<QVPolyline> >())
		draw(	getPropertyValue< QList<QVPolyline> >(name),
			getPropertyValue<QColor>(name + " $$ color"),
			getPropertyValue<bool>(name + " $$ printTags")
			);

	foreach(QString name, getPropertyListByType< QList< QList< QPointF > > >())
		draw(	getPropertyValue< QList<QList< QPointF > > >(name),
			getPropertyValue<QColor>(name + " $$ color"),
			getPropertyValue<bool>(name + " $$ printTags")
			);

	foreach(QString name, getPropertyListByType< QList<QPoint> >())
		draw(	getPropertyValue< QList<QPoint> >(name),
			getPropertyValue<QColor>(name + " $$ color"),
			getPropertyValue<bool>(name + " $$ printTags")
			);

	foreach(QString name, getPropertyListByType< QList<QPointF> >())
		draw(	getPropertyValue< QList<QPointF> >(name),
			getPropertyValue<QColor>(name + " $$ color"),
			getPropertyValue<bool>(name + " $$ printTags")
			);

	custom_viewer();
	qDebug() << "QVImageCanvas::viewer() <~ return";
	}

bool QVImageCanvas::linkProperty(QVWorker &worker, const QString propertyName, QColor color, bool printTags)
	{
	addPropertyFromQVariant(propertyName, inputFlag, worker.getPropertyQVariantValue(propertyName), worker.getPropertyInfo(propertyName));
	addProperty<QColor>(propertyName + " $$ color", outputFlag, color, QString("Color for property ") + propertyName);
	addProperty<bool>(propertyName + " $$ printTags", outputFlag, printTags, QString("Print tag flag ") + propertyName);
	bool result = worker.linkProperty(propertyName, this, propertyName ,QVWorker::AsynchronousLink);
	QObject::connect(&worker, SIGNAL(endIteration(uint, int)), this, SLOT(refreshImageArea()));
	return result;
	}

bool QVImageCanvas::linkROI(QVWorker *worker, const QString propertyName)
	{
	return QVPropertyContainer::linkProperty("rect select", worker, propertyName, QVPropertyContainer::AsynchronousLink);
	}

bool QVImageCanvas::linkPolyline(QVWorker *worker, const QString propertyName)
	{
	return QVPropertyContainer::linkProperty("poly select", worker, propertyName, QVPropertyContainer::AsynchronousLink);
	}

bool QVImageCanvas::unlink(QVWorker *worker, const QString propertyName)
	{
		if (worker->unlinkProperty(propertyName, this, propertyName)) {
			removeProperty(propertyName);
			removeProperty(propertyName + " $$ color");
			removeProperty(propertyName + " $$ printTags");
			QObject::disconnect(worker, SIGNAL(endIteration(uint, int)), this, SLOT(refreshImageArea()));
			return true;
		}
		else if (unlinkProperty("rect select", worker, propertyName))
			return true;
		else if (unlinkProperty("poly select", worker, propertyName))
			return true;

		return false;
	}

void QVImageCanvas::setLowHigh(float low,float high)
	{
	_low = low;
	_high = high;
	}

void QVImageCanvas::draw(const QList<QPoint> &pointList, QColor color, bool printTags)
	{
	getQVPainter()->setPen(color);

	// Paint point as an asterisc.
	for (int i = 0; i< pointList.size(); i++)
		{
		const QPoint point = pointList.at(i);
		if (printTags)
			getQVPainter()->drawText(point + QPoint(2,-2), QString("%1").arg(i));

		getQVPainter()->drawArc(QRect(point + QPoint(-2,-2), point + QPoint(+2,+2)), 0, 16*360);
		}
	}

void QVImageCanvas::draw(const QList<QPointF> &pointList, QColor color, bool printTags)
	{
	getQVPainter()->setPen(color);

	// Paint point as an asterisc.
	for (int i = 0; i< pointList.size(); i++)
		{
		const QPointF last = pointList.at(i);
		if (printTags)
			getQVPainter()->drawText(last+QPointF(2,-2), QString("%1").arg(i));

		getQVPainter()->drawArc(QRectF(last+QPointF(-2,-2),last+QPointF(+2,+2)),0,16*360);
		}
	}

void QVImageCanvas::draw(const QList< QList< QPointF > > &contours, QColor color, bool printTags)
	{
	for (int i = 0; i< contours.size(); i++)
		{
		const QList<QPointF> polyline = contours.at(i);
		
		getQVPainter()->setPen(color);
		getQVPainter()->drawArc(QRectF(polyline.at(0),polyline.at(0)),0,16*360);
		for (int j = 0; j< polyline.size()-1; j++)
			getQVPainter()->drawLine(polyline.at(j)+QPointF(0.5,0.5),polyline.at(j+1)+QPointF(0.5,0.5)); // +0.5 to paint in the center of the pixel.

		// Paint last point as an asterisc.
		//QPoint last = polyline.last();
		const QPointF last = polyline.last();
		if (printTags)
			getQVPainter()->drawText(last+QPoint(2,-2), QString("%1").arg(i));

		getQVPainter()->drawLine(last + QPointF(0.72500,   0.88971), last + QPointF(0.27500,   0.11029));
		getQVPainter()->drawLine(last + QPointF(0.88971,   0.72500), last + QPointF(0.11029,   0.27500));
		getQVPainter()->drawLine(last + QPointF(0.88971,   0.27500), last + QPointF(0.11029,   0.72500));
		getQVPainter()->drawLine(last + QPointF(0.72500,   0.11029), last + QPointF(0.27500,   0.88971));

		//if (polyline.closed)
		//	getQVPainter()->drawLine(polyline.last()+QPointF(0.5,0.5),polyline.at(0)+QPointF(0.5,0.5));
		}
	}

///@todo La posicione de los píxeles esta desplazado respecto a los clic de raton, esto se ve a la hora de pintar polilineas o seleccionar ROIs al máximo zoom, queriendo ajustar la precisión al pixel.
void QVImageCanvas::draw(const QVPolyline &contours, QColor color, bool printTags, TPolyMode mode)
	{
	getQVPainter()->setPen(color);

	for (int i = 0; i< contours.size(); i++)
		{
		const QPoint point = contours.at(i);
		if (printTags)
			getQVPainter()->drawText(point + QPoint(2,-2), QString("%1").arg(i));

		getQVPainter()->drawArc(QRect(point + QPoint(-2,-2), point + QPoint(+2,+2)), 0, 16*360);

		if (mode == LINE)
			if (i > 0)
				// +0.5 to paint in the center of the pixel.
				getQVPainter()->drawLine(contours.at(i-1)+QPointF(0.5,0.5),contours.at(i)+QPointF(0.5,0.5));
			
		}
	}

void QVImageCanvas::draw(const QPoint &center, float radius, QColor color, bool printTags)
	{
	getQVPainter()->setPen(color);

	if (printTags)
		getQVPainter()->drawText(center + QPoint(2,-2), QString("(%1, %2)").arg(center.x(), center.y()));

	int rad = static_cast<int>(radius);
	if (rad > 0) getQVPainter()->drawArc(QRect(center + QPoint(-rad,-rad), center + QPoint(rad,rad)), 0, 16*360);
	}

void QVImageCanvas::draw(const QList<QVPolyline> &contours, QColor color, bool printTags)
	{
	//foreach(QVPolyline polyline, contours)
	for (int i = 0; i< contours.size(); i++)
		{
		const QVPolyline polyline = contours.at(i);
		
		getQVPainter()->setPen(color);
		getQVPainter()->drawArc(QRectF(polyline.at(0),polyline.at(0)),0,16*360);
		for (int j = 0; j< polyline.size()-1; j++)
			getQVPainter()->drawLine(polyline.at(j)+QPointF(0.5,0.5),polyline.at(j+1)+QPointF(0.5,0.5)); // +0.5 to paint in the center of the pixel.

		// Paint last point as an asterisc.
		//QPoint last = polyline.last();
		const QPointF last = polyline.last();
		if (printTags)
			getQVPainter()->drawText(last+QPointF(2,-2), QString("%1").arg(i));

		getQVPainter()->drawLine(last + QPointF(0.72500,   0.88971), last + QPointF(0.27500,   0.11029));
		getQVPainter()->drawLine(last + QPointF(0.88971,   0.72500), last + QPointF(0.11029,   0.27500));
		getQVPainter()->drawLine(last + QPointF(0.88971,   0.27500), last + QPointF(0.11029,   0.72500));
		getQVPainter()->drawLine(last + QPointF(0.72500,   0.11029), last + QPointF(0.27500,   0.88971));

		if (polyline.closed)
			getQVPainter()->drawLine(polyline.last()+QPointF(0.5,0.5),polyline.at(0)+QPointF(0.5,0.5));
		}
	}

void QVImageCanvas::rectSelectedSlot(QRect rect)
	{
	setPropertyValue<QRect>("rect select", rect);
	writeOutputProperties();
	}

///@todo ¿El ROI debe mandarse al worker mientras se está arrastrando, o solo cuando se suente el ratón? -> Hacer concilio
///@todo ¿Cuantos puntos debe contener la polilínea de la circunferencia, uno por pixel o menos? -> Hacer concilio
void QVImageCanvas::circleSelectedSlot(QPoint center, float radius)
	{
	if (radius < 1)
		setPropertyValue<QVPolyline>("poly select", QVPolyline());
	else
		setPropertyValue<QVPolyline>("poly select", QVPolyline::ellipse(static_cast<uInt>(2*PI*radius), center.x(), center.y(), radius, radius, 0.0));

	setPropertyValue<TPolyMode>("poly select $$ polyMode", CIRCLE);
	setPropertyValue<QPoint>("poly select $$ center", center);
	setPropertyValue<float>("poly select $$ radius", radius);
	writeOutputProperties();
	}

void QVImageCanvas::polySelectedSlot(QPoint point, bool reset, TPolyMode mode)
	{
	if (reset)
		setPropertyValue<QVPolyline>("poly select", QVPolyline());
	else
		{
		QVPolyline poly = getPropertyValue<QVPolyline>("poly select");
		poly.append(point);
		setPropertyValue<QVPolyline>("poly select", poly);
		}

	setPropertyValue<TPolyMode>("poly select $$ polyMode", mode);
	writeOutputProperties();
	}

