/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvimagecanvas.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVIMAGECANVAS_H
#define QVIMAGECANVAS_H

#include <QPointF>
#include <QVWorker>
#include <QVPolyline>
#include <qvgui/qvcanvas.h>

/*!
@class QVImageCanvas qvimagecanvas.h qvgui/qvimagecanvas.h
@brief Viewer widget for @ref QVImage objects and other data types.

Displays images from an image source:

@image html qvimagecanvas_penguin.png

For further information about its usage, see section @ref QVImageCanvasWidget.
@ingroup qvgui
*/

class QVImageCanvas: public QVCanvas, public QVPropertyContainer
	{
		Q_OBJECT
	public:
		QVImageCanvas(const QString name = QString (), QWidget *parent=0);
		bool linkProperty(QVWorker &worker, const QString propertyName, QColor color = Qt::red, bool printTags = false);
		bool linkROI(QVWorker *worker, const QString propertyName);
		bool linkPolyline(QVWorker *worker, const QString propertyName);
		bool unlink(QVWorker *worker, const QString propertyName);
		void setLowHigh(float low,float high);
		#ifndef DOXYGEN_IGNORE_THIS
		void viewer();
		#endif
		virtual void custom_viewer() { };

	public slots:
		void rectSelectedSlot(QRect rect);
		void polySelectedSlot(QPoint point, bool reset, TPolyMode mode);
		void circleSelectedSlot(QPoint center, float radius);

	private:
		float _low,_high;
		QPoint circleCenter;
		void draw(const QList<QPoint> &pointList, QColor color = Qt::red, bool printTags = false);
		void draw(const QList<QPointF> &pointList, QColor color = Qt::red, bool printTags = false);
		void draw(const QList< QList< QPointF > > &pointList, QColor color = Qt::red, bool printTags = false);
		void draw(const QVPolyline &contours, QColor color = Qt::red, bool printTags = false, TPolyMode mode = LINE);
		void draw(const QPoint &center, float radius, QColor color = Qt::red, bool printTags = false);
		void draw(const QList<QVPolyline> &contours, QColor color = Qt::red, bool printTags = false);
	};

Q_DECLARE_METATYPE(TPolyMode);

#endif
