/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvmplayercamera.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>

#include <QApplication>
#include <QDebug>
#include <QObject>
#include <QStringList>
#include <QRegExp>
#include <QMutex>

#include <qvipp.h>

#include <QVWorker>
#include <QVMPlayerCamera>

/******************* Auxiliary QVCheckOKMPlayerCamera Class *******************/
QVCheckOKMplayerCamera::QVCheckOKMplayerCamera(QFile & fifo_file,int max_time_ms_to_wait_for_open) : QThread(), _fifo_file(fifo_file), _max_time_ms_to_wait_for_open(max_time_ms_to_wait_for_open)
	{
	qDebug() << "QVCheckOKMplayerCamera::QVCheckOKMplayerCamera()";
	qDebug() << "QVCheckOKMplayerCamera::QVCheckOKMplayerCamera() <- return";
	}

QVCheckOKMplayerCamera::~QVCheckOKMplayerCamera()
	{
	qDebug() << "QVCheckOKMplayerCamera::~QVCheckOKMplayerCamera()";
	quit();
	wait();
	qDebug() << "QVCheckOKMplayerCamera::~QVCheckOKMplayerCamera: thread finished";
	qDebug() << "QVCheckOKMplayerCamera::~QVCheckOKMplayerCamera() <- return";
	}

void QVCheckOKMplayerCamera::run()
	{
	qDebug() << "QVCheckOKMplayerCamera::run()";
	QTimer::singleShot(_max_time_ms_to_wait_for_open, this, SLOT(writeErrorInFifo()));
	qDebug() << "QVCheckOKMplayerCamera::run() <- entering event loop";
	exec();
	qDebug() << "QVCheckOKMplayerCamera::run() <- back from event loop";
	qDebug() << "QVCheckOKMplayerCamera::run() <- return";
	}


void QVCheckOKMplayerCamera::writeErrorInFifo()
	{
	qDebug() << "QVCheckOKMplayerCamera::writeErrorInFifo()";
	_fifo_file.write("MPLAYER ERROR\n"); 
	qDebug() << "QVCheckOKMplayerCamera::writeErrorInFifo() -> return";
	};

/********************* Auxiliary QVCameraThread Class *************************/
QVCameraThread::QVCameraThread(QObject *object,char *slot): QThread(), _object(object), _slot(slot)
	{
	qDebug() << "QVCameraThread::QVCameraThread()";
	_object->moveToThread(this);
	qDebug() << "QVCameraThread::QVCameraThread(): object moved to thread";
	qDebug() << "QVCameraThread::QVCameraThread() <- return";
	}

QVCameraThread::~QVCameraThread()
	{
	qDebug() << "QVCameraThread::~QVCameraThread()";
	disconnect();
	quit();
	qDebug() << "QVCameraThread::~QVCameraThread(): sent quit to thread";
	wait();
	qDebug() << "QVCameraThread::~QVCameraThread(): thread finished";
	qDebug() << "QVCameraThread::~QVCameraThread() <- return";
	}

void QVCameraThread::run()
	{
	qDebug() << "QVCameraThread::run()";
	QTimer *timer = new QTimer();
	timer->start(0);
	connect(timer, SIGNAL(timeout()),_object,_slot);
	qDebug() << "QVCameraThread::run(): timer created, started and connected";
	qDebug() << "QVCameraThread::run(): entering event processing loop";
	exec();
	qDebug() << "QVCameraThread::run(): back from event processing loop";
	delete timer;
	qDebug() << "QVCameraThread::run(): timer deleted";
	qDebug() << "QVCameraThread::run() <- return";
	}

/********************* Auxiliary QVMPlayerIOProcessor class *******************/
QVMPlayerIOProcessor::QVMPlayerIOProcessor(QProcess * mplayer): QObject(), command_queue(), command_queue_mutex()
	{
	qDebug() << "QVMPlayerIOProcessor::QVMPlayerIOProcessor()";
	rows = cols = 0;
	fps = time_length = time_pos = 0;
	speed = 1;
	this->mplayer = mplayer;
	mplayer->waitForReadyRead();
	qDebug() << "QVMPlayerIOProcessor::QVMPlayerIOProcessor() <- return";
	}

QVMPlayerIOProcessor::~QVMPlayerIOProcessor()
	{
	qDebug() << "QVMPlayerIOProcessor::~QVMPlayerIOProcessor()";
	qDebug() << "QVMPlayerIOProcessor::~QVMPlayerIOProcessor() <- return";
	}

void QVMPlayerIOProcessor::queueCommandToMPlayer(const QString &com_str, bool ifEmpty)
	{
	qDebug() << "QVMPlayerIOProcessor::queueCommand("<< com_str <<")";
	qDebug() << "QVMPlayerIOProcessor::queueCommand(): command_queue.size() = "
		<< command_queue.size();
	command_queue_mutex.lock();
	if(!ifEmpty || command_queue.size()==0)
		command_queue << com_str + '\n';
	command_queue_mutex.unlock();
	qDebug() << "QVMPlayerIOProcessor::queueCommand() <- return";
	}

void QVMPlayerIOProcessor::sendCommandToMPlayer()
	{
	qDebug() << "QVMPlayerIOProcessor::sendCommandToMPlayer()";
	command_queue_mutex.lock();
	if (command_queue.size() > 0)
		{
		qDebug() << "QVMPlayerIOProcessor::sendCommandToMPlayer(): sending command:" << command_queue.head();
		mplayer->write(qPrintable(command_queue.dequeue()));
		}
	command_queue_mutex.unlock();
	qDebug() << "QVMPlayerIOProcessor::sendCommandToMPlayer(): wait for bytes written";
	// BUG: In Windows this hangs?
	#ifdef unix
	mplayer->waitForBytesWritten();
	#endif
	qDebug() << "QVMPlayerIOProcessor::sendCommandToMPlayer() <- return";
	}

int QVMPlayerIOProcessor::interpretMPlayerOutput()
	{
	qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput()";
	int length = -1;
	char buf[1024];

	length = mplayer->readLine(buf, sizeof(buf));

	if (length == -1)
		qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): length == -1";
	else	{
		QString str(buf);
		QStringList variables = str.simplified().split("=");
		QStringList palabras = str.simplified().split(" ");

		if(variables[0] == "ANS_speed")
			{
			speed = variables[1].toDouble();
			qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): updating speed =" << speed;
			}
		else if(variables[0] == "ANS_LENGTH")
			{
			time_length = variables[1].toDouble();
			qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): updating time_length =" << time_length;
			}
		else if(variables[0] == "ANS_TIME_POSITION")
			{
			time_pos = variables[1].toDouble();
			qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): updating time_pos =" << time_pos;
			}
		else if (palabras[0] == "VIDEO:")
			{
			// ya tenemos las especificaciones de fps
			for (int i = 0; i < palabras.size(); ++i)
				if (palabras[i] == "fps")
					fps = palabras[i-1].toDouble();

			qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): updating fps" << fps;
			}
		else if (palabras[0] == "VO:")
			{
			// ya tenemos las especificaciones del video y dimensiones
			QStringList dimensiones = palabras[2].split("x");
			cols = dimensiones[0].toInt();
			rows = dimensiones[1].toInt();
			qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): updating cols" << cols;
			qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): updating rows" << rows;
			}
		else
			qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput(): uninterpreted mplayer output:" << str;
		}
	qDebug() << "QVMPlayerIOProcessor::interpretMPlayerOutput() <- return" << length;
	return length;
	}

/******************* Auxiliary QVMPlayerFrameGrabber Class ********************/
QVMPlayerFrameGrabber::QVMPlayerFrameGrabber(QVMPlayerIOProcessor *mplayerIOProcessor, QFile *fifoInput, uInt rows, uInt cols, bool realTime): QObject()
	{
	qDebug() << "QVMPlayerFrameGrabber::QVMPlayerFrameGrabber(fifoInput," << mplayerIOProcessor->rows << "," << mplayerIOProcessor->cols
		<< "," << realTime <<")";
	
	this->mplayerIOProcessor = mplayerIOProcessor;
	this->fifoInput = fifoInput;
	this->realTime = realTime;
	this->finished = false;
	this->frames_read = 0;

	debug_in_memcpy = debug_in_updateSlot = false;

	/// @todo Es necesario inicializar esta imagen?
	imgY = QVImage<uChar>(cols, rows, cols);

	img_auxY = QVImage<uChar>(cols, rows, cols);
	img_auxU = QVImage<uChar>(cols/2, rows/2, cols/2);
	img_auxV = QVImage<uChar>(cols/2, rows/2, cols/2);

	//if (imgY.getCols() != imgY.getStep())
	//	std::cout << "ERROR step " << imgY.getCols() << "\t" << imgY.getStep() << std::endl;
	// ...a YUV420 rows x cols image:
	buf_size = rows*cols + rows*cols/4 + rows*cols/4;

	if (realTime)
		{
		qvcamerathread = new QVCameraThread(this,SLOT(updateSlot()));
		qvcamerathread->start(QThread::NormalPriority);
		}

	qDebug() << "QVMPlayerFrameGrabber::QVMPlayerFrameGrabber() <- return";
	};

QVMPlayerFrameGrabber::~QVMPlayerFrameGrabber()
	{
	qDebug() << "QVMPlayerFrameGrabber::~QVMPlayerFrameGrabber()";
	finished = true;
	if (realTime) delete qvcamerathread;
	qDebug() << "QVMPlayerFrameGrabber::~QVMPlayerFrameGrabber(): thread deleted.";
	qDebug() << "QVMPlayerFrameGrabber::~QVMPlayerFrameGrabber() <- return";
	}

void QVMPlayerFrameGrabber::getNewFrame(QVImage<uChar> &imgY, QVImage<uChar> &imgU, QVImage<uChar> &imgV)
	{
	qDebug() << "QVMPlayerFrameGrabber::getNewFrame(): reading YUV header";
	char buf[1024];
	fifoInput->readLine(buf, sizeof(buf));
	readToBuffer(imgY.getWriteData(), 2*buf_size/3);
	readToBuffer(imgU.getWriteData(), buf_size/6);
	readToBuffer(imgV.getWriteData(), buf_size/6);
	qDebug() << "QVMPlayerFrameGrabber::getNewFrame() <- return";
	}

void QVMPlayerFrameGrabber::updateSlot()
	{
	qDebug() << "QVMPlayerFrameGrabber::updateSlot()";

	qDebug() << "QVMPlayerFrameGrabber::updateSlot(): sent command to mplayer: get_time_pos";
	mplayerIOProcessor->queueCommandToMPlayer("pausing_keep get_time_pos\n",true);

	// Avoid QSocketNotifier annoying message, by deferring call:
	emit sendCommandSignal(); // NOT mplayerIOProcessor->sendCommandToMPlayer();

	// New frame read:
	qDebug() << "QVMPlayerFrameGrabber::updateSlot(): reading frame";
	QVImage<uChar, 1> tempY;
	QVImage<uChar, 1> tempU;
	QVImage<uChar, 1> tempV;

	// We always read in temporal images, that are later safely copied:
	// There is no copy overhead because QVImage uses QSharedData (reference
	// counting).
	getNewFrame(tempY = img_auxY, tempU = img_auxU, tempV = img_auxV);

	// If in real time, we lock, to avoid collision with the thread that is
	// grabbing from the camera:
	if(realTime)
		{
		mutex.lock();
		debug_in_updateSlot = true;
		if (debug_in_memcpy)
			{
			std::cerr << "(BUG1) entered frameUpdate while memcpying!";
			exit(0);
			}
		}

	img_auxY = tempY;
	img_auxU = tempU;
	img_auxV = tempV;

	// Unlock (if in real time):
	if(realTime)
		{
		if (debug_in_memcpy) {
			std::cerr << "(BUG2) entered frameUpdate while memcpying!";
			exit(0);
		}
		debug_in_updateSlot = false;
		condition.wakeAll(); // Wake possible waiting grabs.
		mutex.unlock();
		}

	qDebug() << "QVMPlayerFrameGrabber::updateSlot(): finished" << finished;
	frames_read++;
	qDebug() << "QVMPlayerFrameGrabber::updateSlot(): frames read" << frames_read;

	mplayerIOProcessor->interpretMPlayerOutput();

	qDebug() << "QVMPlayerFrameGrabber::updateSlot() <- return";

	emit newReadFrameGrabber();
	}

void QVMPlayerFrameGrabber::updateFrameBuffer()
	{
	qDebug() << "QVMPlayerFrameGrabber::updateFrameBuffer()";

	// Here we come from a grab, so we always lock (if in real time, it is 
	// obviously needed, but also if not in real time, to make non-real-cameras
	// grabbing also thread-safe (won't be used normally, but who knows...
	// maybe the programmer wish to read different frames from different
	// threads, without discarding any frame as could happen with real time
	// cameras).
	mutex.lock();
	if (debug_in_updateSlot) 
		{
		std::cerr << "(BUG3) entered memcpy while frameUpdating!";
		exit(0);
		}

	if(realTime)
		condition.wait(&mutex); // If in real time, wait for next fresh frame.
	debug_in_memcpy = true;

	// If in real time, updateSlot is asynchronously called by a dedicated
	// thread, otherwise we have to call it explicitly.
	if(not realTime)
		updateSlot();

	imgY = img_auxY;
	imgU = img_auxU;
	imgV = img_auxV;


	if (debug_in_updateSlot) 
		{
		std::cerr << "(BUG4) entered memcpy while frameUpdating!";
		exit(0);
		}
	debug_in_memcpy = false;
	mutex.unlock();

	qDebug() << "QVMPlayerFrameGrabber::updateFrameBuffer() <- return";
	}

void QVMPlayerFrameGrabber::getQVImageYUV(QVImage<uChar> &imgY, QVImage<uChar> &imgU, QVImage<uChar> &imgV) const
	{
	qDebug() << "QVMPlayerFrameGrabber::getQVImageYUV()";
	imgY = this->imgY;
	imgU = this->imgU;
	imgV = this->imgV;
	qDebug() << "QVMPlayerFrameGrabber::getQVImageYUV() <- return";
	}

void QVMPlayerFrameGrabber::getQVImageGray(QVImage<uChar> &imgGray) const
	{
	qDebug() << "QVMPlayerFrameGrabber::getQVImageGray()";
	imgGray = imgY;
	qDebug() << "QVMPlayerFrameGrabber::getQVImageGray() <- return";
	}

void QVMPlayerFrameGrabber::readToBuffer(uChar *buf_img_aux, uInt buf_size)
	{
	uInt totbytes = 0;
	while (totbytes != buf_size and not finished)
		{
		int nbytes = fifoInput->read((char *)(buf_img_aux+totbytes), buf_size-totbytes);

		totbytes += nbytes;
		qDebug() << "QVMPlayerFrameGrabber::readNewFrame(): read" << nbytes << "bytes";

		if(nbytes == 0)
			{
			finished = TRUE;
			qDebug() << "QVMPlayerFrameGrabber::readNewFrame(): read 0 bytes, finishing" << finished;
			break;
			}

		if(nbytes == -1)
			{
			// BUG: problem in Windows, suddenly pipe can not be read ?!!?
			// errorString() -> "Device did not accept the command."
			finished = TRUE;
			qDebug() << "QVMPlayerFrameGrabber::readNewFrame(): error -" << fifoInput->errorString();
			break;
			}			
		}
	}

/***************************** QVMPlayerCamera Class **************************/
QVMPlayerCamera::QVMPlayerCamera(QString name):QVCamera(name), open_options(Default)
	{
	qDebug() << "QVMPlayerCamera::QVMPlayerCamera()";
	if (qvApp == NULL)
		{
		QString str = "QVWorker::QVWorker(): the QVWorker cannot be created before the QVApplication instance. Aborting now.";
		std::cerr << qPrintable(str) << std::endl;
		exit(1);
		}

	frames_grabbed = 0;
	rgbMode = false;
	setStatus(Closed);
	live_camera = FALSE;

	// Properties (to allow reading from argc and argv):
	addProperty<int>("Rows", inputFlag, 0, "Rows to open the camera");
	addProperty<int>("Cols", inputFlag, 0, "Columns to open the camera");
	addProperty<bool>("RealTime", inputFlag, FALSE,"If the camera should be opened in real time mode");
	addProperty<bool>("Deinterlaced", inputFlag, FALSE,"If the camera should be opened in deinterlaced mode");
	addProperty<bool>("NoLoop", inputFlag, FALSE,"If the camera should be opened in no loop mode");
	//addProperty<bool>("RGBMEncoder", inputFlag, FALSE,"If the camera should be opened in RGB using mencoder");
	addProperty<QString>("URL", inputFlag, QString(""),"URL of the video source");
	//addProperty<int>("Count",0,"Test param");

	// QVImage's
	addProperty< QVImage<uChar,3> >("RGB image", outputFlag, QVImage<uChar,3>(), "Last grabbed RGB image");
	addProperty< QVImage<uChar,1> >("Y channel image",outputFlag, QVImage<uChar,1>(), "Last grabbed Y channel image");
	addProperty< QVImage<uChar,1> >("U channel image",outputFlag, QVImage<uChar,1>(), "Last grabbed U channel image");
	addProperty< QVImage<uChar,1> >("V channel image",outputFlag, QVImage<uChar,1>(), "Last grabbed V channel image");

	qDebug() << "QVMPlayerCamera::QVMPlayerCamera() <- return";
	}

QVMPlayerCamera::~QVMPlayerCamera()
	{
	qDebug() << "QVMPlayerCamera::~QVMPlayerCamera()";
	if (status != Closed) closeCam();
	qDebug() << "QVMPlayerCamera::~QVMPlayerCamera() <- deleting image";
	setImageBuffer(NULL);
	qDebug() << "QVMPlayerCamera::~QVMPlayerCamera() <- return";
	}

void QVMPlayerCamera::initMPlayerArgs(QString urlString, uInt rows, uInt cols)
	{
	qDebug() << "QVMPlayerCamera::initMPlayerArgs()";
	qDebug() << "QVMPlayerCamera::initMPlayerArgs(): url string =" << urlString;

	// Loop option and fixed-vo must go first:
	if(not (open_options & NoLoop)) mplayer_args << "-loop" << "0";

	mplayer_args << "-fixed-vo";

	QUrl url(urlString);

	if (url.host() != "")
		path = url.host() + "/";

	path += url.path();

	qDebug() << "QVMPlayerCamera::initMPlayerArgs(): path" << path;

	// Here we infer the type of the video source from the url.
	// If it is not specified in the schema of it, certain match rules are
	// applied to guess it out.
	if (url.scheme() != "")				// schema specified by the user
		schema = url.scheme();
	else if (urlString.startsWith("/dev/video"))	// a v4l device
		schema = "v4l";
	else if (urlString.startsWith("/dev/dv"))	// a dv device
		schema = "dv";
	else if (urlString.contains("*"))		// a multifile
		schema = "mf";
	else if (urlString.startsWith("www."))		// a http file
		schema = "http";
	else if (urlString.startsWith("ftp."))		// a ftp file
		schema = "ftp";

	qDebug() << "QVMPlayerCamera::initMPlayerArgs(): scheme" << schema;

	live_camera = TRUE;	// default configuration

	// Different mplayer args for different kind of image sources:
	if ((schema == "v4l") or (schema == "v4l2") or (schema == "analog"))
		{
		// Video for Linux cameras:
		QString urlQueryValues = QString("driver=%1:device=%2").arg(schema).arg(path);

		QList<QPair<QString, QString> > queryItems = url.queryItems();
		for (int i = 0; i < queryItems.size(); ++i)
			urlQueryValues += ":" + queryItems.at(i).first + "=" + queryItems.at(i).second;

		mplayer_args << "tv://" << "-tv" << urlQueryValues;
		}
	else if (schema == "dv")
		// DV cameras (connected through firewire):
		mplayer_args << path << "-demuxer" << "rawdv" << "-cache" << "400";
	else if (schema == "iidc")
		// IIDC cameras (connected through firewire):
		// For example, iidc:///dev/video1394/0?from=/dev/video0:to=/dev/video1
		qFatal("Currently this driver does not work (apparently with\n"
			"vloopback writing and then reading from a fifo with mplayer\ndoes not work).\n");
	else if (schema == "tv")
		// Analog TV input:
		qFatal("tv URL: Still not implemented\n");
	else if (schema == "dvb")
		// Digital TV input:
		qFatal("dvb URL: Still not implemented\n");
	else	// Format can be rtsp:// http://, ftp://, dvd://, vcd://, mf:// or file://
		{ 
		//  We pass the url it directly to mplayer:
		live_camera = FALSE;
		if (schema != "")
			mplayer_args << QString(schema + "://" + path);
		else
			mplayer_args << path;
		}

	// IMPORTANT!! All -vf filters MUST be together, separated by commas, and in
	// the right order. By now, we only use deinterlacing and scale, and in that order:
	QString aux;

	// Deinterlace option:
	if(open_options & Deinterlaced) aux = "pp=md";

	if(rows != 0 or cols != 0)
		{
		if(aux != QString())
			aux += QString(",scale=%1:%2").arg(cols).arg(rows);
        	else
			aux = QString("scale=%1:%2").arg(cols).arg(rows);
        	}

	if (aux != QString()) mplayer_args << "-vf" << aux;

	// Real time option:
	if(not (open_options & RealTime))
		{
		if(not live_camera)
        		mplayer_args << "-benchmark";
		else
			// The user attempted to open a live camera in continuous mode.
			// We issue a warning:
			qWarning("Warning: opening live cameras in continuous mode "
					 "wastes less CPU time, but it is prone to delays in "
					 "the pipe of images if your process is slower than "
					 "the camera.");
		}

	// Additional arguments (slave, quiet, nosound & yuv4mpeg output):
	mplayer_args << "-slave" << "-quiet" << "-nosound" << "-vo" << QString("yuv4mpeg:file=%1").arg(namedPipe->getInputFilePath());

	qDebug() << "QVMPlayerCamera::initMPlayerArgs() <- return";
	}

// (Implicit) default open method for a camera (uses introspection to read 
// parameters, allowing initialization from argc and argv):
bool QVMPlayerCamera::openCam()
	{
	OpenOptions optsProp = Default;

	readInputProperties();

	if (getPropertyValue<bool>("RealTime"))		optsProp |= RealTime;
	if (getPropertyValue<bool>("Deinterlaced"))	optsProp |= Deinterlaced;
	if (getPropertyValue<bool>("NoLoop"))		optsProp |= NoLoop;
	if (rgbMode)					optsProp |= RGBMEncoder;

	return openCam(getPropertyValue<QString>("URL"), getPropertyValue<int>("Rows"), getPropertyValue<int>("Cols"), optsProp);
	}

// Explicit open url (and possible options) method:
bool QVMPlayerCamera::openCam(const QString & urlstring,OpenOptions opts)
	{ return openCam(urlstring,0,0,opts); }

// Main method to open a camera:
bool QVMPlayerCamera::openCam(const QString & urlstring, uInt rows, uInt cols, OpenOptions opts)
	{
	qDebug() << "QVMPlayerCamera::openCam(" << urlstring << "," << rows << "," << cols << "," << opts << ")";

	setStatus(Closed);

	open_options = opts;

	// Pipe initialization
	namedPipe = new QNamedPipe(QString("mplayer"));
	fifo_file.setFileName(namedPipe->getOutputFilePath());

	initMPlayerArgs(urlstring, rows, cols);

	qDebug() << "QVMPlayerCamera::openCam(): MPlayer args ->" << mplayer_args;
	mplayer.start(MPLAYER_BINARY_PATH,mplayer_args);
	qDebug() << "QVMPlayerCamera::openCam(): after mplayer.start()";
	if(not mplayer.waitForStarted(1000))
		qFatal("Mplayer failed to start: Are you sure it is installed and in the correct PATH?");
	qDebug() << "QVMPlayerCamera::openCam(): after mplayer.waitForstarted()";
	
	// We create a guarding thread that will unblock the subsequent
	// fifo_file.readLine in the case that mplayer does not start OK (due,
	// for example, to a wrong URL, an incorrect file format, or whatever).
	// If mplayer starts OK before two seconds (for local videos) or ten
	// seconds (for DVD, VCD and remote videos), this thread silently stops and
	// is deleted.
	QVCheckOKMplayerCamera *check_thread;
	if (schema == "http" or schema == "ftp" or schema == "rtsp" or schema == "dvd" or schema == "vcd")
		check_thread = new QVCheckOKMplayerCamera(fifo_file, 10000);
	else
		check_thread = new QVCheckOKMplayerCamera(fifo_file, 2000);

	check_thread->moveToThread(check_thread);
	check_thread->start();
	qDebug() << "QVMPlayerCamera::openCam(): after starting QVCheckOKMplayerCamera thread";

	// Important to open in ReadWrite mode, to avoid infinite waiting if
	// mplayer did not start OK, and because perhaps the former guarding
	// thread will write in the FIFO.
	qDebug() << "QVMPlayerCamera::openCam(): opening fifo_file";
	if((fifo_file.open(QIODevice::ReadWrite|QIODevice::Unbuffered)) == -1)
		qFatal("Error opening fifo %s\n", qPrintable(namedPipe->getPipeName()));
	qDebug() << "QVMPlayerCamera::openCam(): reading fifo_file";

	// Now we read YUV header:
	char buf[1024];
	int len;
	if ((len = fifo_file.readLine(buf, sizeof(buf))) == -1) return FALSE;
	if (QString(buf) == QString("MPLAYER ERROR\n")) { 
		std::cerr << "QVMPlayerCamera::openCam(): Warning: Mplayer could not open the requested video source (" << qPrintable(urlstring) << ") of type " << qPrintable(schema) <<std::endl;
		check_thread->quit();
		check_thread->wait();
		delete check_thread;

		// Cleaning: mplayer.waitForFinished();
		mplayer.terminate();
		if (not mplayer.waitForFinished(500)) mplayer.kill();	
		fifo_file.close();
		delete namedPipe;

		qDebug() << "QVMPlayerCamera::openCam(): QVCheckOKMplayerCamera thread deleted after detecting an error; returning FALSE";
		return FALSE;
	}

	check_thread->quit();
	check_thread->wait();

	delete check_thread;
	qDebug() << "QVMPlayerCamera::openCam(): QVCheckOKMplayerCamera thread deleted after correct launch of mplayer";

	qDebug() << "QVMPlayerCamera::openCam(): going to create QVMPlayerIOProcessor...";
	mplayerIOProcessor = new QVMPlayerIOProcessor(&mplayer);

	qDebug() << "QVMPlayerCamera::openCam(): waiting for initialization.";
	while(getRows() == 0 || getCols() == 0)
		mplayerIOProcessor->interpretMPlayerOutput();

 	qDebug() << "QVMPlayerCamera::openCam(): video dims (" << getCols() << "," << getRows() << ")";
	qDebug() << "QVMPlayerCamera::openCam(): fps " << getFPS();

	mplayerIOProcessor->queueCommandToMPlayer("get_time_length");

	setImageBuffer(new QVImage<uChar,1>(getRows(), getCols(), getCols()));
	mplayerFrameGrabber = new QVMPlayerFrameGrabber(mplayerIOProcessor, &fifo_file, getRows(), getCols(), /*false,*/ open_options & QVMPlayerCamera::RealTime);

	connect(mplayerFrameGrabber,SIGNAL(newReadFrameGrabber()),this,SIGNAL(newRead()));

	// These signals are to defer the call to sendCommandToMplayer, and avoid
	// the "QSocketNotifier:: socket notifier cannot be enabled from another
	// thread" warning.
	connect(mplayerFrameGrabber,SIGNAL(sendCommandSignal()),mplayerIOProcessor,SLOT(sendCommandToMPlayer()));
	connect(this,SIGNAL(sendCommandSignal()),mplayerIOProcessor,SLOT(sendCommandToMPlayer()));

	setStatus(Running);

	qDebug() << "QVMPlayerCamera::openCam() <- return";

	emit camOpened();

	//grab();

	return TRUE;
	}

// Close camera method:
void QVMPlayerCamera::closeCam()
	{
	qDebug() << "QVMPlayerCamera::closeCam()";

	if (status == Closed)
		{
		qDebug() << "QVMPlayerCamera::closeCam(): camera already closed. Returning";
		qDebug() << "QVMPlayerCamera::closeCam() <- return";
		return;
		}

	setStatus(Closed);

	emit camClosed();

	mplayerIOProcessor->queueCommandToMPlayer("quit");	
	// Avoid QSocketNotifier annoying message, by deferring call:
	emit sendCommandSignal(); // NOT mplayerIOProcessor->sendCommandToMPlayer();

	delete mplayerFrameGrabber;
	qDebug() << "QVMPlayerCamera::closeCam(): mplayerFrameGrabber deleted";

	delete mplayerIOProcessor;
	qDebug() << "QVMPlayerCamera::closeCam(): mplayerIOProcessor deleted";

	mplayer.terminate();
	while (not mplayer.waitForFinished(500)) mplayer.kill();
	qDebug() << "QVMPlayerCamera::closeCam(): mplayer terminated";
	
	fifo_file.close();
	qDebug() << "QVMPlayerCamera::closeCam(): closed fifo_file";
	
	delete namedPipe;
	qDebug() << "QVMPlayerCamera::closecam(): deleted namedpipe";

	qDebug() << "QVMPlayerCamera::closeCam() <- return";
	}

bool QVMPlayerCamera::grab()
	{
	qDebug() << "QVMPlayerCamera::grab()";

	if(!performGrab())
		return false;

	QVImage<uChar> grabbedYImage, grabbedUImage, grabbedVImage;
	mplayerFrameGrabber->getQVImageYUV(grabbedYImage, grabbedUImage, grabbedVImage);

	setPropertyValue< QVImage<uChar,1> >("Y channel image", grabbedYImage);
	setPropertyValue< QVImage<uChar,1> >("U channel image", grabbedUImage);
	setPropertyValue< QVImage<uChar,1> >("V channel image", grabbedVImage);

	if (open_options & QVMPlayerCamera::RGBMEncoder)
		{
		QVImage<uChar,3> grabbedRGBImage(grabbedYImage.getCols(),grabbedYImage.getRows());
		YUV420ToRGB(grabbedYImage, grabbedUImage, grabbedVImage, grabbedRGBImage);
		setPropertyValue< QVImage<uChar,3> >("RGB image", grabbedRGBImage);
		}
	/*else	{
		//QVImage<uChar> grabbedYImage, grabbedUImage, grabbedVImage;
		//mplayerFrameGrabber->getQVImageYUV(grabbedYImage, grabbedUImage, grabbedVImage);
		setPropertyValue< QVImage<uChar,1> >("Y channel image", grabbedYImage);
		setPropertyValue< QVImage<uChar,1> >("U channel image", grabbedUImage);
		setPropertyValue< QVImage<uChar,1> >("V channel image", grabbedVImage);
		}*/

	writeOutputProperties();

	qDebug() << "QVMPlayerCamera::grab() <~ return";

	return true;
	}

bool QVMPlayerCamera::grab(QVImage<uChar> &imgY, QVImage<uChar> &imgU, QVImage<uChar> &imgV)
	{
	qDebug() << "QVMPlayerCamera::grab(imageY, imageU, imageV)";

	bool newImages = performGrab();

	if (newImages)
		mplayerFrameGrabber->getQVImageYUV(imgY, imgU, imgV);

	setPropertyValue< QVImage<uChar,1> >("Y channel image", imgY);
	setPropertyValue< QVImage<uChar,1> >("U channel image", imgU);
	setPropertyValue< QVImage<uChar,1> >("V channel image", imgV);

	qDebug() << "QVMPlayerCamera::grab() <~ return";

	return newImages;
	}

bool QVMPlayerCamera::grab(QVImage<uChar,3> & imageRGB)
	{
	qDebug() << "QVMPlayerCamera::grab(imageRGB)";
	bool newImages = performGrab();

	QVImage<uChar> grabbedYImage, grabbedUImage, grabbedVImage;

	if (newImages)
		{
		mplayerFrameGrabber->getQVImageYUV(grabbedYImage, grabbedUImage, grabbedVImage);

		imageRGB = QVImage<uChar,3>(grabbedYImage.getCols(),grabbedYImage.getRows());
		YUV420ToRGB(grabbedYImage, grabbedUImage, grabbedVImage, imageRGB);
		}

	setPropertyValue< QVImage<uChar,3> >("RGB image", imageRGB);
	
	qDebug() << "QVMPlayerCamera::grab() <~ return";
	return newImages;
	}

bool QVMPlayerCamera::grab(QVImage<uChar,1> & imageGray)
	{
	qDebug() << "QVMPlayerCamera::grab(imageGray)";

	bool newImages = performGrab();

	if (newImages)
		mplayerFrameGrabber->getQVImageGray(imageGray);

	setPropertyValue< QVImage<uChar,1> >("Y channel image", imageGray);

	qDebug() << "QVMPlayerCamera::grab() <~ return";
	return newImages;
	}

bool QVMPlayerCamera::performGrab()
	{
	// First, if there are any pending commands, we send them to mplayer:
	qDebug() << "QVMPlayerCamera::performGrab()";

	if (isClosed()) return false;

	qDebug() << "QVMPlayerCamera::performGrab(): status != Closed";

	bool newFrameGrabbed = false;
	// This is for when the camera is paused; it will return the same frames
	// over and over, but not immediately, but at a maximum fps rate (to avoid
	// intense CPU usage, that will slow the rest of the application and the
	// GUI.

	switch(status)
		{
		case RunningOneStep:
			qDebug() << "QVMPlayerCamera::performGrab(): status == RunningOneStep";
			setStatus(Paused);
		case Running:
			qDebug() << "QVMPlayerCamera::performGrab(): status == Running";
			mplayerFrameGrabber->updateFrameBuffer();
			frames_grabbed++;
			newFrameGrabbed = true;
			qDebug() << "QVMPlayerCamera::performGrab(): frames grabbed" << getFramesGrabbed();
			break;

		case Paused:
			qDebug() << "QVMPlayerCamera::performGrab(): status == Paused";
			/*
			mutexJustToSleepMiliseconds.lock();
			qDebug() << "QVMPlayerCamera::performGrab(): pausing" << milisecsToWait
				<< "milisecs (video is "<< getFPS() <<" FPS's)";
			conditionJustToSleepMiliseconds.wait(&mutexJustToSleepMiliseconds,milisecsToWait);
			qDebug() << "QVMPlayerCamera::performGrab(): unlocking";
			mutexJustToSleepMiliseconds.unlock(); */
			break;

		default:
			break;
		}
	qDebug() << "QVMPlayerCamera::performGrab(): checking finished";

	if (mplayerFrameGrabber->finished)
		closeCam();
	else
		emit newGrab();

	qDebug() << "QVMPlayerCamera::performGrab() <- return";

	return newFrameGrabbed;
	}

// Pause, unpause, next frame, set speed, seek and close camer slots:
// In all of them, the corresponding command is sent to the shared command queue
// data structure, so access is accordingly protected .
void QVMPlayerCamera::pauseCam()
	{
	qDebug() << "QVMPlayerCamera::pauseCam()";
	if(status == Closed) return;
	setStatus(Paused);
	qDebug() << "QVMPlayerCamera::pauseCam() <- return";
	}

void QVMPlayerCamera::unpauseCam()
	{
	qDebug() << "QVMPlayerCamera::unpauseCam()";
	if(status == Closed) return;
	setStatus(Running);
	qDebug() << "QVMPlayerCamera::unpauseCam() <~ return";
	}

void QVMPlayerCamera::nextFrameCam()
	{
	qDebug() << "QVMPlayerCamera::nextFrameCam()";
	if(status == Closed) return;
	setStatus(RunningOneStep);
	qDebug() << "QVMPlayerCamera::nextFrameCam() <~ return";
	}

void QVMPlayerCamera::setSpeedCam(double d)
	{
	qDebug() << "QVMPlayerCamera::setSpeedCam()";
	if(status == Closed) return;
	mplayerIOProcessor->queueCommandToMPlayer(QString("pausing_keep speed_set ") + QString::number(d));
	mplayerIOProcessor->queueCommandToMPlayer("get_property speed");
	qDebug() << "QVMPlayerCamera::setSpeedCam() <~ return";
	}

void QVMPlayerCamera::seekCam(TSeekType seek,double d)
	{
	qDebug() << "QVMPlayerCamera::seekCam()";
	if(status == Closed) return;
	if(status == Paused)
		setStatus(RunningOneStep);
	QString command = QString("pausing_keep seek ") + QString::number(d) + " " + QString::number(seek);
	mplayerIOProcessor->queueCommandToMPlayer(command);
	qDebug() << "QVMPlayerCamera::seekCam() <~ return";
	}

bool QVMPlayerCamera::link(QVWorker* worker, const QString imageName)
	{
	if(worker->isType<QVImage<uChar,1> >(imageName))
		{
		connect(worker, SIGNAL(startIteration()), this, SLOT(grab()), Qt::DirectConnection);
		return this->linkProperty("Y channel image", worker, imageName, AsynchronousLink);
		}
	else if(worker->isType<QVImage<uChar,3> >(imageName))
		{
		rgbMode = true;
		connect(worker, SIGNAL(startIteration()), this, SLOT(grab()), Qt::DirectConnection);
		return this->linkProperty("RGB image", worker, imageName, AsynchronousLink);
		}
	else
		{
		std::cerr << "Warning: QVMPlayerCamera::link(): unsupported property type: property " << qPrintable(imageName)
			<< " in holder " << qPrintable(worker->getName()) << "is not a QVImage<uChar,1> or QVImage<uChar,3>" << std::endl;
		return FALSE; //Should never get here;
		}
	}

bool QVMPlayerCamera::link(QVWorker* worker, const QString imageNameY, const QString imageNameU, const QString imageNameV)
	{
	if(!worker->isType< QVImage<uChar,1> >(imageNameY))
		{
		std::cerr << "Warning: QVMPlayerCamera::link(): image " << qPrintable(imageNameY) << "is not of type QVImage<uChar,1> in holder "
			<< qPrintable(worker->getName()) << std::endl;
		return FALSE;
		}

	if(!worker->isType< QVImage<uChar,1> >(imageNameU))
		{
		std::cerr << "Warning: QVMPlayerCamera::link(): image " << qPrintable(imageNameU) << "is not of type QVImage<uChar,1> in holder "
			<< qPrintable(worker->getName()) << std::endl;
		return FALSE;
		}

	if(!worker->isType< QVImage<uChar,1> >(imageNameV))
		{
		std::cerr << "Warning: QVMPlayerCamera::link(): image " << qPrintable(imageNameV) << "is not of type QVImage<uChar,1> in holder "
			<< qPrintable(worker->getName()) << std::endl;
		return FALSE;
		}

	connect(worker, SIGNAL(startIteration()), this, SLOT(grab()), Qt::DirectConnection);
	return	this->linkProperty("Y channel image", worker, imageNameY, AsynchronousLink) &&
		this->linkProperty("U channel image", worker, imageNameU, AsynchronousLink) &&
		this->linkProperty("V channel image", worker, imageNameV, AsynchronousLink);
	}

bool QVMPlayerCamera::unlink(QVWorker* worker, const QString imageName)
	{
		disconnect(worker, SIGNAL(startIteration()), this, SLOT(grab()));

		if (rgbMode) {
			return unlinkProperty("RGB image", worker, imageName);
		}
		else {
			return unlinkProperty("Y channel image", worker, imageName);
		}
	}

