/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvmultidimensionalminimizer.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVMULTIDIMENSIONALFUNCTIONMINIMIZER_H
#define QVMULTIDIMENSIONALFUNCTIONMINIMIZER_H

#include <qvmath.h>
#include <QVVector>

#include <gsl/gsl_errno.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_multimin.h>

#ifndef DOXYGEN_IGNORE_THIS
/*!
@class QVMultidimensionalMinimizer qvmultidimensionalminimizer.h qvmath/qvmultidimensionalminimizer.h
@brief Wrapper class for GSL multidimensional function minimization.

@todo document.
@ingroup qvfunctionoptimization
*/

class QVMultidimensionalMinimizer
	{
	private:
		int iterations, vectorSize;
		QVVector actual, step;
		double minimumValue;

		static double static_function (const gsl_vector *v, void *multidimensionalOptimizerPtr);

	public:	
		virtual double function (const QVVector &v) = 0;

		QVMultidimensionalMinimizer(const int vectorSize, const double stepSize):
			iterations(0), vectorSize(vectorSize), actual(vectorSize,0), step(vectorSize, stepSize)	{ }

		void setStartingVector(const QVVector &vector)	{ actual = vector; }
		void setStepVector(const QVVector &vector)	{ step = vector; }
		void setMinimum(const QVVector &vector)		{ actual = vector; }
		void setMinimumValue(const double value)	{ minimumValue = value; }

		const QVVector & getMinimum()		const	{ return actual; }
		const double & getMinimumValue()	const	{ return minimumValue; }
		const int & getIterations()		const	{ return iterations; }

		const QList<QVVector> gridIterate(const QVVector &firstVertex, const QVVector &secondVertex, const int divisions = 5, const int maxIterations = 100, const double maxError = 1e-5);
		bool iterate(const int maxIterations = 100, const double maxError = 1e-5);
	};

#endif
#endif

